// Copyright (C) 2013 by IAdea. All Rights Reserved

function parseIcs(data) {
    var icsLines = data.split('\n');

    var ics;
    var state = 'INI';
    var iEvent = 0, iTz = 0;

    for (var i in icsLines) {
        var icsLine = icsLines[i].replace(/\r$/gm, '');

        var _nv = icsLine.split(':');
        var nv = [];
        nv[0] = _nv.shift();
        if (_nv.length >= 1) {
            nv[1] = _nv.join(":");
        }

        if (nv.length == 2) {
            if (nv[0].indexOf(';') >= 0) {
                var breakPlace = icsLine.indexOf(';');
                nv[0] = icsLine.substr(0, breakPlace);
                nv[1] = icsLine.substr(breakPlace);
            }

            switch (state) {
                case 'INI':
                    if ('BEGIN' == nv[0] && 'VCALENDAR' == nv[1]) {
                        state = 'CAL';
                        ics = {};
                    }
                    else
                        state = 'ERR';
                    break;

                case 'CAL':
                    if ('BEGIN' == nv[0]) {
                        switch (nv[1]) {
                            case 'VTIMEZONE':
                                state = 'TZN';
                                ics['TIMEZONE'] = ics['TIMEZONE'] || [];
                                ics['TIMEZONE'][iTz] = {};
                                break;
                            case 'VEVENT':
                                state = 'EVT';
                                ics['EVENT'] = ics['EVENT'] || [];
                                ics['EVENT'][iEvent] = {};
                                break;
                            default:
                                state = 'ERR';
                                break;
                        }
                    }
                    else
                        ics[nv[0]] = nv[1];

                    break;

                case 'TZN':
                    switch (nv[0]) {
                        case 'END':
                            if ('VTIMEZONE' == nv[1]) {
                                state = 'CAL';
                                iTz++;
                            }
                            else
                                state = 'ERR';
                            break;
                        case 'BEGIN':
                            switch (nv[1]) {
                                case 'STANDARD':
                                    state = 'CST';
                                    ics['TIMEZONE'][iTz]['STANDARD'] = {};
                                    break;
                                case 'DAYLIGHT':
                                    state = 'CDL';
                                    ics['TIMEZONE'][iTz]['DAYLIGHT'] = {};
                                    break;
                                default:
                                    state = 'ERR';
                            }
                            break;
                        default:
                            ics['TIMEZONE'][iTz][nv[0]] = nv[1];
                    }
                    break;

                case 'CST':
                    if ('END' == nv[0]) {
                        if ('STANDARD' == nv[1])
                            state = 'TZN';
                        else
                            state = 'ERR';
                    }
                    else
                        ics['TIMEZONE'][iTz]['STANDARD'][nv[0]] = nv[1];

                    break;

                case 'CDL':
                    if ('END' == nv[0]) {
                        if ('DAYLIGHT' == nv[1])
                            state = 'TZN';
                        else
                            state = 'ERR';
                    }
                    else
                        ics['TIMEZONE'][iTz]['DAYLIGHT'][nv[0]] = nv[1];

                    break;

                case 'EVT':
                    if ('END' == nv[0]) {
                        if ('VEVENT' == nv[1]) {
                            state = 'CAL';
                            iEvent++;
                        }
                        else
                            state = 'ERR';
                    }
                    else
                        ics['EVENT'][iEvent][nv[0]] = nv[1];

                    break;
            }
        }

        if ('ERR' == state)
            break;
    }

    //console.log('ics = ', ics);
    return ics;
}

function getIsoDateString(spec, timezones) {
    var parsed = /(?:;TZID=(.*):)?(\d{8}T\d{6})(Z)?$/.exec(spec);

    if (parsed) {
        var tz = parsed[1];
        var tzSpec;
        if (tz) {
            var now_y = parsed[2].substr(0, 4);
            var now_M = parsed[2].substr(4, 2);
            var now_d = parsed[2].substr(6, 2);
            var now_h = parsed[2].substr(9, 2);
            var now_m = parsed[2].substr(11, 2);
            var now_s = parsed[2].substr(13, 2);
            var target_d = new Date(parseInt(now_y), parseInt(now_M) - 1, parseInt(now_d), parseInt(now_h), parseInt(now_m), parseInt(now_s));
            
            var isDstObserved = target_d.isDstObserved();
            for (var iTz in timezones) {
                if (timezones[iTz].TZID == tz) {
                    // FIXME: handle DST here
                    /*
                    var useDaylight = false;  
                    if (timezones[iTz].STANDARD && timezones[iTz].STANDARD.DTSTART && timezones[iTz].DAYLIGHT && timezones[iTz].DAYLIGHT.DTSTART) {
                        if (timezones[iTz].DAYLIGHT.DTSTART > timezones[iTz].STANDARD.DTSTART) {
                            //----- standard ------ daylight ------
                            if (parsed[2] < timezones[iTz].STANDARD.DTSTART || parsed[2] >= timezones[iTz].DAYLIGHT.DTSTART) {
                                //use daylight
                                useDaylight = true;
                            }
                        }
                        else if (timezones[iTz].DAYLIGHT.DTSTART < timezones[iTz].STANDARD.DTSTART) {
                            //----- daylight ------ standard ------
                            if (parsed[2] >= timezones[iTz].DAYLIGHT.DTSTART && parsed[2] < timezones[iTz].STANDARD.DTSTART) {
                                //use daylight
                                useDaylight = true;
                            }
                        }
                    }
                    */
                    if (isDstObserved) {
                        tzSpec = timezones[iTz].DAYLIGHT.TZOFFSETFROM;
                        if (timezones[iTz].DAYLIGHT.TZOFFSETTO) {
                            tzSpec = timezones[iTz].DAYLIGHT.TZOFFSETTO;
                        }
                    }
                    else {
                        tzSpec = timezones[iTz].STANDARD.TZOFFSETFROM;
                        if (timezones[iTz].STANDARD.TZOFFSETTO) {
                            tzSpec = timezones[iTz].STANDARD.TZOFFSETTO;
                        }
                    }

                    tzSpec = tzSpec.substr(0, 3) + ':' + tzSpec.substr(3, 2);
                }
            }
        }

        var rawDate = parsed[2];
        var isoDate = rawDate.substr(0, 4) + '-' + rawDate.substr(4, 2) + '-' + rawDate.substr(6, 2)
            + 'T' + rawDate.substr(9, 2) + ':' + rawDate.substr(11, 2) + ':' + rawDate.substr(13, 2)
            + (tzSpec ? tzSpec : 'Z');

        return isoDate;
    }
    else
        return null;
}

Date.prototype.stdTimezoneOffset = function () {
    var jan = new Date(this.getFullYear(), 0, 1);
    var jul = new Date(this.getFullYear(), 6, 1);
    return Math.max(jan.getTimezoneOffset(), jul.getTimezoneOffset());
}

Date.prototype.isDstObserved = function () {
    return this.getTimezoneOffset() < this.stdTimezoneOffset();
}

function getCurrNextSessions(events) {
    events.sort(function (a, b) {
        return a.start - b.start;
    });

    var dtNow = new Date();

    var iCurr = -1, iNext = -1;

    for (var i in events) {
        if (dtNow >= events[i].start && dtNow <= events[i].end)
            iCurr = i;
        else if (events[i].start > dtNow && -1 == iNext)
            iNext = i;
    }

    return { 'iCurr': iCurr, 'iNext': iNext };
}

function formatBriefDT(dt) {
    var bShowSec = false;

    var year = dt.getYear() + 1900,
        month = dt.getMonth(),
        day = dt.getDate(),
        dow = dt.getDay(),
        hour = dt.getHours() % 12,
        minute = dt.getMinutes(),
        second = dt.getSeconds(),
        isPm = (dt.getHours() >= 12);

    if (minute < 10) minute = '0' + minute;
    if (second < 10) second = '0' + second;
    if (0 == hour) hour = 12;

    return ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'][dow] + ' '
        + ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][month]
        + ' ' + day + ', ' + year + ' ' + hour + ':' + minute + (bShowSec ? ':' + second : '')
        + (isPm ? 'pm' : 'am');
}

function formatDuration(ms) {
    var bShowSec = true;

    var output;

    var seconds = Math.floor(ms / 1000);

    if (seconds < 60 && bShowSec)
        output = seconds + ' s';
    else {
        var minutes = Math.floor(seconds / 60);
        seconds %= 60;

        if (minutes < 60)
            output = minutes + ' m ' + (bShowSec ? seconds + ' s' : '');
        else {
            var hours = Math.floor(minutes / 60);
            minutes %= 60;

            if (hours < 24)
                output = hours + ' h ' + minutes + ' m '
                    + (bShowSec ? seconds + ' s' : '');
            else {
                var days = Math.floor(hours / 24);
                hours %= 24;
                output = days + 'd ' + hours + 'h ' + minutes + 'm '
                    + (bShowSec ? seconds + 's' : '');
            }
        }
    }

    return output;
}
